import GraphSettings
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

from Averaged_QMMM_vs_DFT_MD import add_core_plot
from plot_tools import plot_averaged_barrier, plot_H_at_core, add_lines_to_core
from scipy.interpolate import interp1d
from ase.io import read
from matplotlib import lines

def add_core_H_plot(fig, i, rect,
                    H_glide_averaged_images,
                    bulk,
                    H_glide_core_positions,
                    legend=False,
                    legend_box=(0.25, -1.0, 0.5, 0.5),
                    spine_color="black", spine_lw=2.0,
                    xlim=(-2.0, 5.5)):

    core1 = fig.add_axes(rect)

    core_x, core_y = H_glide_core_positions.T

    f_core = interp1d(core_x, core_y, kind='cubic')
    xnew_core = np.linspace(core_x.min(), core_x.max(), num=41, endpoint=True)

    H_positions = []
    images = H_glide_averaged_images
    H_z_values = []
    Nknots = len(images)

    for image in images:

        image.get_chemical_symbols()
        symbols = np.array(image.get_chemical_symbols())

        H = image[symbols == "H"].copy()

        H.wrap()

        H_x, H_y, H_z = H.positions[0]
        H_positions.append((H_x, H_y))
        H_z_values.append(H_z)

    H_z_distance = H_z_values - H_z_values[0]
    b = images[0].cell[2][2]

    H_positions = np.array(H_positions)

    #print(b)
    #print(H_z_distance)

    for i_H, H_z_d_val in enumerate(H_z_distance):

        if H_z_d_val > b:

            H_z_distance[i_H] -= b

        elif H_z_d_val < 0.:

            H_z_distance[i_H] += b

    b = 3.17 * np.sqrt(3.0) / 2.

    size = 150 - 200 * H_z_distance / b

    H_x, H_y = H_positions.T

    f_H = interp1d(H_x, H_y, kind='cubic')
    xnew_H = np.linspace(H_x.min(), H_x.max(), num=41, endpoint=True)

    H_image = H_glide_averaged_images[i]

    plot_H_at_core(H_image,
                   bulk,
                   ax=core1,
                   imp_size=0.75*size[i], xyscale=3, insert=True)


    core1.plot(xnew_H, f_H(xnew_H), linestyle="dashed", color="C4", lw=2)

    core1.scatter(core_x, core_y, marker="^", facecolor="none", edgecolor="C3", s=15)
    core1.plot(xnew_core, f_core(xnew_core), linestyle="dashed", color="C3",
               lw=2, label="core trajectory")

    core1.scatter(H_glide_core_positions.T[0][i],
                  H_glide_core_positions.T[1][i], marker="^", color="C3",
                  s=150, zorder=14)

    core1.scatter(H_x, H_y, marker="d", facecolor="none", edgecolor="C4", s=0.1*size)


    core1.set_xlim(xlim)
    core1.set_ylim((-1.8, 1.5))

    #core1.scatter(core_x, core_y, marker="^", facecolor='None', s=15, edgecolor="C3")
    if legend:
        core1.legend(bbox_to_anchor=legend_box, fontsize=10, loc="lower center")

    core1.set_aspect("equal")

    core1.set_xticks([])
    core1.set_yticks([])
    core1.set_ylabel("")
    core1.set_xlabel("")
    core1.set_title("")

    #core1.set_axis_off()

    for axis in ['top','bottom','left','right']:
        core1.spines[axis].set_linewidth(spine_lw)
        core1.spines[axis].set_color(spine_color)


    return core1

if __name__ == '__main__':

    GraphSettings.Apply_thesis_settings()

    H_glide_averaged_images = read("H_glide_averaged_images.xyz", index=":")
    H_glide_core_positions = np.loadtxt("H_glide_core_positions.txt")
    bulk = read("QMMM_bulk.xyz")

    pure_glide_averaged_images = read("pure_glide_averaged_images.xyz", index=":")
    QM_MM_core_positions = np.loadtxt("QMMM_core_positions.txt")
    pure_glide_dataframe = pd.read_csv("pure_glide_averaged_energies.csv")
    QM_MM_bulk = read("QMMM_bulk.xyz")

    x, y, z = bulk.positions.T

    bulk.positions -= np.array((x.mean() - 2.0 * 3.17 * np.sqrt(6.)/6.0,
                                y.mean() + 0.5, 0.0))

    for image in H_glide_averaged_images:

        image.positions += np.array((3.17 * 2.0*np.sqrt(6.)/6.0,
                                     -0.5,
                                     0.0))

    H_glide_dataframe = pd.read_csv("H_glide_averaged_energies.csv")

    fig = plt.figure(figsize=(8, 6))
    ax = fig.add_subplot(111)

    s_H, E_H, QMMM_H = plot_averaged_barrier(H_glide_averaged_images,
                                             H_glide_dataframe,
                                             label="glide with H", ms=10,
                                             ax=ax, s_norm=True, color="C1",
                                             marker="^")

    s_W, E_W, QMMM = plot_averaged_barrier(pure_glide_averaged_images, pure_glide_dataframe,
                                         label="glide in pure W", ms=7.5,
                                         ax=ax, s_norm=True, s_shift=0.5)

    ax.set_xticks([0.0, 0.50, 1.0, 1.5])

    ax.set_xticklabels(["split", "easy", "split", "easy"])
    ax.set_xlabel('Screw dislocation core type')

    ax.set_xlim(-0.05, 1.55)
    ax.set_ylim(-0.04, 0.132)

    topax = ax.twiny()
    topax.set_xlabel("Normalised NEB coordinate")
    topax.set_xlim(ax.get_xlim())
    topax.set_xticks(ax.get_xticks())



    x_h_start = -1.8
    x_start = -3.1
    x_width = 7.5

    rect = (-0.025, 0.11, 0.3, 0.15)
    i = 0
    x_line_start = -3.25 #+ 3.17*np.sqrt(3)/6.0 + 0.4

    first_H_core = add_core_H_plot(fig, i, rect,
                                   H_glide_averaged_images,
                                   bulk,
                                   H_glide_core_positions,
                                   spine_color="C1", spine_lw=2.0,
                                   xlim=(x_line_start, x_line_start + 6.5))

    rect = (0.22, 0.11, 0.3, 0.15)
    i = 0

    first_pure_core = add_core_plot(fig, i, rect,
                                    pure_glide_averaged_images,
                                    QM_MM_bulk,
                                    QM_MM_core_positions,
                                    core_size=150, spine_color="C0",
                                    spine_lw=2.0, core_lw=2.0,
                                    xlim=(x_start, x_start + x_width))

    rect = (0.22, 0.39, 0.3, 0.15)
    i = 3
    second_H_core = add_core_H_plot(fig, i, rect,
                                    H_glide_averaged_images,
                                    bulk,
                                    H_glide_core_positions,
                                    spine_color="C1",
                                    xlim=(x_h_start, x_h_start + x_width))

    rect = (0.65, 0.11, 0.3, 0.15)
    i = -1
    last_H_core = add_core_H_plot(fig, i, rect,
                                  H_glide_averaged_images,
                                  bulk,
                                  H_glide_core_positions,
                                  spine_color="C1", legend=True,
                                  legend_box=(0.61, 2.8, 0.5, 0.5),
                                  xlim=(x_h_start, x_h_start + x_width))

    rect = (0.49, 0.67, 0.3, 0.15)
    i = 3

    second_pure_core = add_core_plot(fig, i, rect,
                                     pure_glide_averaged_images,
                                     QM_MM_bulk,
                                     QM_MM_core_positions,
                                     core_size=150, spine_color="C0",
                                     spine_lw=2.0, core_lw=2.0,
                                     xlim=(x_start, x_start + x_width))

    shift_E = 0.011
    shift_s = 0.25

    rect = (0.735, 0.37, 0.3, 0.15)
    i = -1
    x_width = 4.0*3.17*np.sqrt(6)/6.0 + 1.2
    x_start = -0.6#-3.17*np.sqrt(6)/6.0

    last_pure_core = add_core_plot(fig, i, rect,
                                   pure_glide_averaged_images,
                                   QM_MM_bulk,
                                   QM_MM_core_positions,
                                   core_size=150, spine_color="C0",
                                   spine_lw=2.0, core_lw=2.0,
                                   xlim=(x_start, x_start + x_width))

    ax.grid(True, linestyle="dashed")

    ax.set_ylabel('Energy (eV)', labelpad=0)

    proxy_QMMM_H = ax.errorbar([-10], [-10], fmt="C1^-", yerr=[0], ms=7.5,
                               linewidth=2.0, label="glide with H")
    proxy_QMMM = ax.errorbar([-10], [-10], fmt="C0o-", yerr=[0], ms=7.5,
                             linewidth=2.0, label="glide in pure W")

    ax.legend(loc="upper left", handles=[proxy_QMMM_H, proxy_QMMM],
              fontsize=12)


    ax.set_yticks(np.linspace(0, 0.12, 5))


    add_lines_to_core(ax, first_pure_core, s_W[0], E_W[0],
                      orientation="top",
                      color="C0")

    add_lines_to_core(ax, second_pure_core, s_W[3], E_W[3],
                      orientation="bottom",
                      color="C0")

    add_lines_to_core(ax, last_pure_core, s_W[-1], E_W[-1],
                      orientation="bottom",
                      color="C0", x_shift=0.085)

    add_lines_to_core(ax, first_H_core, s_H[0], E_H[0],
                      orientation="top",
                      color="C1", x_shift=0.085)

    add_lines_to_core(ax, second_H_core, s_H[3], E_H[3],
                      orientation="top",
                      color="C1")

    add_lines_to_core(ax, last_H_core, s_H[-1], E_H[-1],
                      orientation="left",
                      color="C1")


    #plt.show()
    fig.savefig("Averaged_H_glide.pdf")
