import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import os

from ase.io import read
from plot_tools import *
from scipy.optimize import curve_fit
from scipy.interpolate import interp1d

import GraphSettings

import matplotlib.patches as mpatches

class AnyObject(object):
    pass

class Lines(object):
    def legend_artist(self, legend, orig_handle, fontsize, handlebox):
        x0, y0 = handlebox.xdescent, handlebox.ydescent
        width, height = handlebox.width, handlebox.height
        xy = np.array(((x0, y0),
                       (x0 + width, y0)))
        line = mpatches.Polygon(xy, color="C1", linestyle="dashed",
                                    lw=2, transform=handlebox.get_transform())

        handlebox.add_artist(line)
        xy = np.array(((x0 , y0 + 0.48 * height),
                       (x0 + width, y0 + 0.48 * height)))

        line = mpatches.Polygon(xy, color="C2", linestyle="dashed",
                                    lw=2, transform=handlebox.get_transform())

        handlebox.add_artist(line)

        xy = np.array(((x0 , y0 +  height),
                       (x0 + width, y0 + height)))

        line = mpatches.Polygon(xy, color="C4", linestyle="dashed",
                                    lw=2, transform=handlebox.get_transform())

        handlebox.add_artist(line)

        return None

GraphSettings.Apply_thesis_settings()

dft_alat = 3.17
dft_Ecoh = -8.48
dft_C11 = 541
dft_C12 = 204
dft_C44 = 142

ini_QMMM_images = []

xyzfiles = [xyzfile for xyzfile in os.listdir("ini_relaxed_images_QMMM/") if "imageH" in xyzfile]

for xyzfile in xyzfiles:
    ini_QMMM_images.append(read("ini_relaxed_images_QMMM/" + xyzfile))

ini_core_positions_QMMM = np.loadtxt('ini_relaxed_images_QMMM/core_positions.txt')

pure_glide_averaged_images = read("pure_glide_averaged_images.xyz", index=":")
QM_MM_bulk = read("QMMM_bulk.xyz")

QM_MM_core_positions = np.loadtxt("QMMM_core_positions.txt")

core_x, core_y = QM_MM_core_positions.T

f_core = interp1d(core_x, core_y, kind='cubic')
xnew_core = np.linspace(core_x.min(), core_x.max(), num=41, endpoint=True)

i_values = [13, 7, 15]

n = (len(i_values) + 1) / 2


fig = plt.figure(figsize=(3.0*4, 3.0*2))

left_pad = 0.1
spacing = 0.02

height = (0.99 - (left_pad + spacing))/2.0
width = height/2.0

rect = (left_pad/2.0,
        left_pad + height + spacing,
        width, height)
ax1 = fig.add_axes(rect)

rect = ((left_pad + height + spacing)/2.0,
        left_pad + height + spacing,
        width, height)
ax2 = fig.add_axes(rect)

rect = (left_pad/2.0,
        left_pad,
        width, height)
ax3 = fig.add_axes(rect)

rect = ((left_pad + height + spacing)/2.0,
        left_pad,
        width, height)
ax4 = fig.add_axes(rect)

left_axes = [ax1, ax2, ax3, ax4]

right_pad = 0.13

rect = (0.5 + right_pad/2.0,
        left_pad,
        0.5 - right_pad,
        0.7 - left_pad)

right_ax = fig.add_axes(rect)

H_size = 100

plot_H_at_core(pure_glide_averaged_images[0],
               QM_MM_bulk, imp_size=H_size,
               core_position=QM_MM_core_positions[0],
               ax=ax1, title="", imp_symbol="H", insert=True)


core_x, core_y = QM_MM_core_positions[-1]

ax1.scatter(core_x, core_y, s=150, marker="^",
                  # edgecolor="C3", lw=2,
                   c="C1",
                   zorder=10, label="final core position")

W = ax1.scatter([], [], marker="o", c="white", s=150, edgecolors="k", label="W atoms")
core_x, core_y = QM_MM_core_positions[0]

core1 = ax1.scatter(core_x, core_y, s=150, marker="^",
                      color="C5", zorder=10, label="Initial SD core position")

core2 = ax1.scatter([], [], s=150, marker="^",
                      color="C1", label="Final SD core position")

H = ax1.scatter([], [], marker="d", c="C4", s=150,
                       label='H atoms')

handles = [core1, core2]

ax1.legend(loc="upper right", handles=handles,
          ncol=1, bbox_to_anchor=(1.0, 1.0))


center = (0.0, 0.0, 0.0)

imp_SD_tetras = sd.screw_cyl_tetrahedral(dft_alat, dft_C11, dft_C12, dft_C44,
                                         scan_r=4.5, symbol="H",
                                         center=center)

x, y, z = imp_SD_tetras.positions.T

mask = (y > -1.15) & (y < 0.95) & (x > -2.15) & (x < 4.55)
mask2 = (y > -3.15) & (y < -1.15) & (x > -0.75) & (x < 3.0)

imp_SD_tetras = imp_SD_tetras[np.logical_or(mask, mask2)]

x, y, z = imp_SD_tetras.positions.T

ax1.scatter(x, y, c="C0", marker="d", s=H_size, edgecolor="k")

ax1.set_xlim((-2.5, 5.5))
ax1.set_ylim((-4, 4))

for i, ax, imp_color in zip(i_values, left_axes[1:], ["C1", "C6", "C9"]):

    plot_H_at_core(ini_QMMM_images[i], QM_MM_bulk,
                   core_position=ini_core_positions_QMMM[i],
                   ax=ax, title="", imp_size=H_size, imp_color=imp_color,
                   insert=True)

    ax.set_xlim((-2.5, 5.5))
    ax.set_ylim((-4, 4))


W = ax4.scatter([], [], marker="o", c="white", s=100, edgecolors="k", label="W atoms")
H = ax4.scatter([], [], marker="d", c="white", s=100, edgecolors="k",
                       label='H atoms')

core_trajectory, = ax4.plot([], [], "C3--",
                              linewidth=2.0, label="Core glide trajectory in pure W")

relaxed_core = ax4.scatter([], [], color="C3", marker="^", s=150,
                       label='Relaxed core position')

handles = [W, H, core_trajectory, relaxed_core]

ax4.legend(loc="upper right", handles=handles,
           ncol=2, bbox_to_anchor=(0.925, 1.0))


for ax, label in zip(left_axes, ["(a)", "(b)", "(c)", "(d)"]):

    ax.text(-1.75, -2.25, label, fontsize=20)

for ax in [ax2, ax3, ax4]:
    ax.plot(xnew_core, f_core(xnew_core), linestyle="dashed", color="C3", lw=2)

for ax in left_axes:
    ax.set_xlabel("")
    ax.set_ylabel("")

for ax in [ax1, ax2]:
    ax.set_xticks([])

for ax in [ax2, ax4]:
    ax.set_yticks([])

fig.text(0.25, 0.0025, "<112> coordinate ($\mathrm{\AA}$)",  ha='center')
fig.text(0.0025, 0.5, "<110> coordinate ($\mathrm{\AA}$)",
         va='center', rotation='vertical')

ax = right_ax

pure_glide_averaged_images = read("pure_glide_averaged_images.xyz", index=":")
averaged_images_H = read("H_positions_NEB/in_core_4_8_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_incore_4_8.csv")

#print(final_dataframe_H)
__, __, in1 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                       label="In core 1", s_norm=True, ax=ax, color="C1", markersize=7.5)

averaged_images_H = read("H_positions_NEB/in_core_5_8_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_incore_5_8.csv")

__, __, in2 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                                    label="In core 2", s_norm=True,
                                    ax=ax, color="C1",
                                    markersize=7.5)

averaged_images_H = read("H_positions_NEB/out_core_3_5_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_outcore_3_5.csv")

__, __, out1 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                      label="Out core 1", s_norm=True, ax=ax, color="C2",
                                     marker="^", markersize=7.5)

averaged_images_H = read("H_positions_NEB/out_core_8_9_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_outcore_8_9.csv")

__, __, out2 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                      label="Out core 2", s_norm=True, ax=ax, color="C4",
                                     marker="s", markersize=7.5)



proxy_in1 = ax.errorbar([-10], [-10], fmt="C1o-", yerr=[0],
                        linewidth=2.0, label="In core", markersize=7.5)
proxy_out1 = ax.errorbar([-10], [-10], fmt="C2^-", yerr=[0],
                         linewidth=2.0, label="Out core 1", markersize=7.5)
proxy_out2 = ax.errorbar([-10], [-10], fmt="C4s-", yerr=[0],
                         linewidth=2.0, label="Out core 2", markersize=7.5)


ax.legend(loc="upper left", bbox_to_anchor=(-0.175, 1.365),
          handles=[proxy_in1, proxy_out1, proxy_out2],
          title="Migration barriers")

ax.set_ylim((-0.035, 0.2))
ax.set_xlim((-0.05, 1.15))

ax.set_xlabel("Normalised NEB coordinate")
ax.set_ylabel("Energy (eV)")

ax.set_yticks([0.0, 0.05, 0.10, 0.15])

ax.text(0.1, 0.125, "(e)", fontsize=20)

rect = (0.825, 0.65, 0.15, 0.3)

core_ax = fig.add_axes(rect)
QM_MM_bulk = read("QMMM_bulk.xyz")
plot_vitek(pure_glide_averaged_images[0], QM_MM_bulk, xyscale=5, plot_axes=core_ax)
#ax.set_xlabel("<112> coordinate, $\AA$")
#ax.set_ylabel("<110> coordinate, $\AA$")

b = 3.17 * np.sqrt(3) / 2.

H_images = read("H_relaxed_positions.xyz")
cluster_id = H_images.get_array("cluster_id")

colors = ["k", "C5", "C6", "C1", "C9"]

x, y, z = H_images.positions.T

for c_id, color in zip(range(int(cluster_id.max()) + 1), colors):

    mask = cluster_id == c_id
    core_ax.scatter(x[mask], y[mask], c=color, marker="d", s=100,
                    label="H positions", edgecolors="k")

H = core_ax.scatter([], [], marker="d", c="white", edgecolors="k", label="H atoms", s=150)
W = core_ax.scatter([], [], marker="o", c="white", edgecolors="k", label="W atoms", s=150)
core, = ax.plot([], [], "C3^-", markersize=5, label="core position")

core_ax.legend(bbox_to_anchor=(-1.025, 0.2175, 1.0, 1.0),
               handles=[W, H, core, AnyObject()],
               handler_map={AnyObject: Lines()},
               labels=["W atoms",
                       "H atoms",
                       "Core position",
                       "H trajectories"],
               fontsize=12)

in_core_images = read("H_positions_NEB/in_core_5_8_fin_images.xyz",
                      index=":")

in_core_2_images = read("H_positions_NEB/in_core_8_4_fin_images.xyz",
                        index=":")

out_core_images = read("H_positions_NEB/out_core_8_9_fin_images.xyz",
                       index=":")

out_core_2_images = read("H_positions_NEB/out_core_3_5_fin_images.xyz",
                         index=":")[::-1]

neb_images = [in_core_images, in_core_2_images]

#ax.plot((-b/2. - 0.125, b/2. + 0.125), (-4.95, -4.95), c="C1", lw=2)
#ax.plot((-b/2. - 0.125, b/2. + 0.125), (4.95, 4.95), c="C1", lw=2)

#fig.savefig("H_positions_along_x_y.eps")

for images in neb_images:

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    core_ax.plot(Hx, Hy, "C1--", lw=2)

neb_images = [out_core_images, out_core_2_images]

colors2 = ["C2", "C4"]
for images, color in zip(neb_images, colors2):

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    core_ax.plot(Hx, Hy, color + "--", lw=2)

core_ax.scatter(0.0, 0.0, marker="^", c="C3")

core_ax.set_xlim((-2.2, 2.2))
core_ax.set_ylim((-2.4, 2.0))

core_ax.set_xticks([])
core_ax.set_yticks([])

core_ax.set_aspect("equal")
core_ax.yaxis.set_label_position("right")

core_ax.text(-1.9, -1.9, "(f)", fontsize=20)

rect = (0.85, 0.435, 0.1, 0.2)

core_ax2 = fig.add_axes(rect)
core_ax2.set_xticks([])
core_ax2.set_yticks([])

colors = ["C5", "C6", "C1", "C9"]

for c_id, color in zip([1, 2, 3, 4], colors):

    mask = cluster_id == c_id
    core_ax2.scatter(x[mask], z[mask], c=color, marker="d", s=115,
                     label="H positions", edgecolors="k", zorder=10)

b = 3.17*np.sqrt(3.0)*0.5

core_ax2.set_ylim((-0.625, b + 0.125))
core_ax2.set_xlim((-b/2. - 0.375, b/2. + 0.375))

core_ax2.set_aspect("equal")

line = core_ax2.axvline(0, c='r', label="Dislocation line")

#core_ax2.legend(loc="best", handles=[line])
core_ax2.set_aspect('equal')

#Wx, Wy, Wz = QM_MM_bulk.positions.T
ini_dislo = read("ini_scaled_disloc_relaxed.xyz")
ini_dislo.wrap()
Wx, Wy, Wz = ini_dislo.positions.T

mask1 = np.logical_and(Wx < b/2. + 0.375, Wx > -b/2. - 0.375)
mask2 = np.logical_and(Wy < 2.0, Wy > -2.4)
mask = np.logical_and(mask1, mask2)

core_ax2.scatter(Wx[mask], Wz[mask],
                 color=["red", "lime", "blue"],
                 edgecolor="k", s=225, lw=1)

core_ax2.scatter(Wx[mask], Wz[mask] - b,
                 color=["red", "lime", "blue"],
                 edgecolor="k", s=225, lw=1)

core_ax2.scatter(Wx[mask], Wz[mask] + b,
                 color=["red", "lime", "blue"],
                 edgecolor="k", s=225, lw=1)

core_ax2.text(-1.75, 2.2, "(g)", fontsize=20)

neb_images = [in_core_images, in_core_2_images]

for images in neb_images:

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    core_ax2.plot(Hx, Hz, "C1--", lw=2, zorder=15)

neb_images = [out_core_images, out_core_2_images]

colors2 = ["C2", "C4"]
for images, color in zip(neb_images, colors2):

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    core_ax2.plot(Hx, Hz, color + "--", lw=2, zorder=15)

core_ax2.set_aspect('equal')
core_ax2.set_xlabel("<112>")
core_ax2_twinx = core_ax2.twinx()
core_ax2_twinx.set_yticks(())
core_ax2_twinx.set_ylabel("<111>")

core_ax.set_ylabel("<110>")

core_ax_twiny = core_ax.twiny()
core_ax_twiny.set_xticks(())
core_ax_twiny.set_xlabel("<112>")

#plt.show()

fig.savefig("Core_structure_with_H_relaxed_with_barrier.pdf")
