import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import GraphSettings

from plot_tools import *
from ase.io import read

GraphSettings.Apply_thesis_settings()

fig, ax = plt.subplots(figsize=(6,6))

pure_glide_averaged_images = read("pure_glide_averaged_images.xyz", index=":")
averaged_images_H = read("H_positions_NEB/in_core_4_8_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_incore_4_8.csv")

"""
__, __, in1 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                       label="In core 1", s_norm=True, ax=ax, color="C1")
"""
averaged_images_H = read("H_positions_NEB/in_core_5_8_averaged_images.xyz",
                         index=":")


final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_incore_5_8.csv")
"""
__, __, in2 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                      label="In core 2", s_norm=True, ax=ax, color="C1")
"""

averaged_images_H = read("H_positions_NEB/out_core_3_5_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_outcore_3_5.csv")

E_shift_3_5 = np.r_[final_dataframe_H['E']][-1]
r_3_5 = np.r_[final_dataframe_H['s']]
r_3_5 /= r_3_5[-1]

r1, fie1, out1 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                      label="Out core 1", s_norm=True, ax=ax, color="C2")

H_pos_3_5 = np.zeros((len(averaged_images_H),2))
for i,images in enumerate(averaged_images_H):
    H_mask = np.array(images.get_chemical_symbols()) == "H"
    H_pos_3_5[i] = images.get_positions()[H_mask][0][:-1]

from plot_tools import plot_averaged_barrier

averaged_images_H = read("H_positions_NEB/out_core_8_9_averaged_images.xyz",
                         index=":")

final_dataframe_H = pd.read_csv("H_positions_NEB/1000_MC_sampled_av_energies_outcore_8_9.csv")

E_shift_8_9 = np.r_[final_dataframe_H['E']][-1]

r_8_9 = np.r_[final_dataframe_H['s']]
r_8_9 /= r_8_9[-1]


r2, fie2, out2 = plot_averaged_barrier(averaged_images_H, final_dataframe_H,
                      label="Out core 2", s_norm=True, ax=ax, color="C4")

H_pos_8_9 = np.zeros((len(averaged_images_H),2))
for i,images in enumerate(averaged_images_H):
    H_mask = np.array(images.get_chemical_symbols()) == "H"
    H_pos_8_9[i] = images.get_positions()[H_mask][0][:-1]



""" ELASTIC BIND ENERGY PLOTS """

x, y, E = np.loadtxt("elastic_interaction_energies/H_elasticE_3_5.dat", unpack=True)



R = np.sqrt((x-x[0])**2 + (y-y[0])**2)
DeltaR = (R - R[0])

E_spl = interp1d(r_3_5,E,'cubic')
rr = np.linspace(0.,1.,51)
oce1, = ax.plot(rr, E_spl(rr)-E_spl(rr[-1])+E_shift_3_5, "C2-", lw=3, label='Out core 1 (An. El)')

x, y, E = np.loadtxt("elastic_interaction_energies/H_elasticE_8_9.dat", unpack=True)

E_spl = interp1d(r_8_9,E,'cubic')
rr = np.linspace(0.,1.,51)
oce2, = ax.plot(rr, E_spl(rr)-E_spl(rr[-1])+E_shift_8_9, "C4-", lw=3, label='Out core 2 (An. El)')

""" END ELASTIC """

ax.legend(loc="lower right", handles=[out1, oce1, out2, oce2])

ax.set_ylim((-0.235, 0.235))
ax.set_xlim((-0.05, 1.15))


ax.set_yticks([-0.2,-0.15,-0.1,-0.05,0.0, 0.05, 0.10, 0.15, 0.20])

ax.text(0.08, 0.16, "(a)", fontsize=22)

fig.tight_layout()
fig.savefig("H_at_SD_migration_barrier_e_only.pdf", dpi=80)
