import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

from ase.io import read
from matscipy import dislocation as sd

import GraphSettings
from plot_tools import plot_vitek

GraphSettings.Apply_thesis_settings()

fig, ax, = plt.subplots(1, 1, squeeze=True, figsize=(6, 6))

QM_MM_bulk = read("QMMM_bulk.xyz")
bulk=QM_MM_bulk

pure_glide_averaged_images = read("pure_glide_averaged_images.xyz", index=":")

plot_vitek(pure_glide_averaged_images[0], bulk, xyscale=5, plot_axes=ax)
ax.set_xlabel("<112> coordinate ($\AA$)")
ax.set_ylabel("<110> coordinate ($\AA$)")

b = 3.17 * np.sqrt(3) / 2.

H_images = read("H_relaxed_positions.xyz")
cluster_id = H_images.get_array("cluster_id")

colors = ["k", "C5", "C6", "C1", "C9"]

x, y, __ = H_images.positions.T

for c_id, color in zip(range(int(cluster_id.max()) + 1), colors):

    mask = cluster_id == c_id
    ax.scatter(x[mask], y[mask], c=color, marker="d", s=225,
               label="H positions", edgecolors="k", zorder=2)

H = ax.scatter([], [], marker="d", c="white", edgecolors="k", label="H atoms", s=200)
W = ax.scatter([], [], marker="o", c="white", edgecolors="k", label="W atoms", s=200)
core = ax.scatter([0.0], [0.0], c="r", marker="^", s=50, zorder=3, label="SD core")

ax.legend(loc="upper center", handles=[W, H, core], fontsize=15,
          bbox_to_anchor=(0.25, 0.65, 0.5, 0.5))

in_core_images = read("H_positions_NEB/in_core_5_8_fin_images.xyz",
                      index=":")

in_core_2_images = read("H_positions_NEB/in_core_8_4_fin_images.xyz",
                        index=":")

out_core_images = read("H_positions_NEB/out_core_8_9_fin_images.xyz",
                       index=":")

out_core_2_images = read("H_positions_NEB/out_core_3_5_fin_images.xyz",
                         index=":")[::-1]

neb_images = [in_core_images, in_core_2_images]

ax.plot((-b/2. - 0.125, b/2. + 0.125), (-4.95, -4.95), c="C1", lw=2)
ax.plot((-b/2. - 0.125, b/2. + 0.125), (4.95, 4.95), c="C1", lw=2)

#fig.savefig("H_positions_along_x_y.eps")

for images in neb_images:

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    ax.plot(Hx, Hy, "C1--", lw=3)

neb_images = [out_core_images, out_core_2_images]

colors2 = ["C2", "C4"]
for images, color in zip(neb_images, colors2):

    H_mask = np.array(images[0].get_chemical_symbols()) == "H"
    H_positions = np.array([image.get_positions()[H_mask] for image in images])
    H_positions = H_positions.reshape((len(images), 3))


    Hx, Hy, Hz = H_positions.T
    ax.plot(Hx, Hy, color + "--", lw=3)

x = np.loadtxt("elastic_interaction_energies/Screw_PD_Interaction_x.dat")
y = np.loadtxt("elastic_interaction_energies/Screw_PD_Interaction_y.dat")
E = np.loadtxt("elastic_interaction_energies/Screw_PD_Interaction_E.dat")

im = ax.imshow(-E.T, cmap="bwr", interpolation="hermite",
               vmin=-0.175, vmax=0.175, extent=[x.min(),
                                                x.max(),
                                                y.min(),
                                                y.max()])

ax.scatter([0.0], [0.0], c="lightgrey", s=1000, zorder=1)
ax.scatter([0.0], [0.0], c="r", marker="^", s=50, zorder=3)
cbar = fig.colorbar(im, ax=ax, label="Interaction energy (eV)")


ax.set_xlim(x.min(),x.max())
ax.set_ylim(y.min(),y.max())

ax.text(-2.5, 3.2, "(b)", fontsize=22)

fig.savefig("H_positions_along_x_y_traj_elastic.pdf")
