import numpy as np
import matplotlib.pyplot as plt
import sys

import matscipy.dislocation as sd
import shapely.geometry as sg
import descartes

from ase.lattice.cubic import BodyCenteredCubic
from matplotlib.legend_handler import HandlerLine2D
from matplotlib.patches import FancyArrowPatch, Circle
import matplotlib.patches as mpatches

import GraphSettings
from plot_tools import plot_vitek

GraphSettings.Apply_thesis_settings()

xy = np.array(((0, 0), (0.5, 0.5), (1.0, 0.0)))
triangle = mpatches.Polygon(xy, color='red', label='The red data')

class AnyObject(object):
    pass

class Triangles(object):
    def legend_artist(self, legend, orig_handle, fontsize, handlebox):
        x0, y0 = handlebox.xdescent, handlebox.ydescent
        width, height = handlebox.width, handlebox.height
        xy = np.array(((x0, y0),
                       (x0 + 0.2 * width, y0 + height),
                       (x0 + 0.4 * width, y0)))
        triangle = mpatches.Polygon(xy, color="C3",
                                    lw=2, transform=handlebox.get_transform())

        handlebox.add_artist(triangle)
        xy = np.array(((x0 + 0.6 * width, y0),
                       (x0 + 0.8 * width, y0 + height),
                       (x0 + width, y0)))
        triangle = mpatches.Polygon(xy, facecolor='none', edgecolor='C3',
                                    lw=2, transform=handlebox.get_transform())

        handlebox.add_artist(triangle)
        return patch

def plot_bulk(ax,
              ylim=(-2.0, 2.0),
              xlim=(-2.0, 2.0)):


    ymin, ymax = ylim

    Wx_bulk, Wy_bulk, Wz_bulk = bulk_ini_demo.positions.T

    bulk_mask = np.logical_and((Wy_bulk > ymin), (Wy_bulk < ymax ))

    total_mask = np.logical_and(red_mask, bulk_mask)

    ax.scatter(Wx_bulk[total_mask],
               Wy_bulk[total_mask],
               color="red",
               edgecolor="k", s=size, lw=1)

    total_mask = np.logical_and(green_mask, bulk_mask)

    ax.scatter(Wx_bulk[total_mask],
               Wy_bulk[total_mask],
               color="lime",
               edgecolor="k", s=size, lw=1)

    total_mask = np.logical_and(blue_mask, bulk_mask)

    ax.scatter(Wx_bulk[total_mask],
               Wy_bulk[total_mask],
               color="blue",
               edgecolor="k", s=size, lw=1)

    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    ax.set_aspect("equal")
    ax.set_xlabel("<112>")
    ax.set_ylabel("<100>")

    return None

def plot_dipole(disloc_quadrupole, shift, displacement, ax, net=False):
    x, y, z = (disloc_quadrupole.positions + shift).T
    dislo_coord_left2 = dislo_coord_left + shift
    sc = ax.scatter(x, y, c=displacement, cmap="coolwarm", vmax=1.2, vmin=-1.2)
    dislo_coord_right2 = dislo_coord_right + shift
    ax.scatter(dislo_coord_left2[0], dislo_coord_left2[1], marker="^", c="C3")
    ax.scatter(dislo_coord_right2[0], dislo_coord_right2[1], marker="v", c="C3")

    if net:
        ax.axvline(dislo_coord_left2[0], linestyle="dashed", c="g")
        ax.axvline(dislo_coord_right2[0], linestyle="dashed", c="g")
        ax.axhline(dislo_coord_left2[1], linestyle="dashed", c="g")

    return None

def plot_bulk_z(ax, xlim=(-1.65, 1.65)):


    xmin, xmax = xlim

    Wx_bulk, Wy_bulk, Wz_bulk = bulk_ini_demo.positions.T

    bulk_mask = np.logical_and((Wy_bulk > xmin), (Wy_bulk < xmax ))

    for b_val in [-b, 0, b]:

        total_mask = np.logical_and(red_mask, bulk_mask)

        ax.scatter(Wx_bulk[total_mask],
                   Wz_bulk[total_mask] + b_val,
                   color="red",
                   edgecolor="k", s=size, lw=1)

        total_mask = np.logical_and(green_mask, bulk_mask)

        ax.scatter(Wx_bulk[total_mask],
                   Wz_bulk[total_mask] + b_val,
                   color="lime",
                   edgecolor="k", s=size, lw=1)

        total_mask = np.logical_and(blue_mask, bulk_mask)

        ax.scatter(Wx_bulk[total_mask],
                   Wz_bulk[total_mask] + b_val,
                   color="blue",
                   edgecolor="k", s=size, lw=1)

    ax.set_xlim(xlim)
    ax.set_aspect("equal")
    ax.set_xlabel("<112>")
    ax.set_ylabel("<111>")

    return None

size = 200

alat = 3.17
n1u = 5

W = BodyCenteredCubic(directions=[[1, -2, 1],
                                  [2, -1, -1],
                                  [1, 1, 1]],
                      symbol='W',
                      pbc=(True, True, True),
                      latticeconstant=alat, debug=0)

unit_cell_u, unit_cell_v, unit_cell_z = W.get_cell()


disloc_quadrupole, \
bulk, \
dislo_coord_left, \
dislo_coord_right = sd.make_screw_quadrupole(alat, n1u=n1u)

final_u = disloc_quadrupole.positions - bulk.positions
x_scaled, y_scaled, __ = bulk.get_scaled_positions(wrap=False).T
burgers = bulk.cell[2][2]
u_tilt = 0.5 * burgers * (y_scaled - 0.5)
final_u.T[2] += u_tilt

shift = 0.0
mode = 1.0

shifted_positions = bulk.positions + shift - dislo_coord_left
x, y, __ = shifted_positions.T
displacement_left = np.arctan2(y, x) * burgers / (2.0 * np.pi)

shifted_positions = bulk.positions + shift - dislo_coord_right
x, y, __ = shifted_positions.T
displacement_right = np.arctan2(y, mode*x) * burgers / (2.0 * np.pi)

core_x_shift = 2.0 * alat * np.sqrt(6.)/6.0
core_distance = (dislo_coord_right - dislo_coord_left)[0]

fig, ax = plt.subplots(figsize=(15.0, 6.0), nrows=1,
                       sharex=True, tight_layout=True)

ax1 = fig.add_axes((0.06, 0.025, 0.25, 0.5))

core_distance = (dislo_coord_right - dislo_coord_left)[0]
print(core_distance)
cylinder_r = core_distance / 2.0
print(cylinder_r)


dft_alat = 3.17
dft_Ecoh = -8.48
dft_C11 = 541
dft_C12 = 204
dft_C44 = 142

b = 0.5*dft_alat*np.sqrt(3)

cyl_disloc, cyl_bulk, u = sd.make_screw_cyl(alat,
                                            dft_C11,
                                            dft_C12,
                                            dft_C44,
                                            cylinder_r=cylinder_r, cutoff=5.5)

disloc_fin, disloc_ini, bulk_ini = sd.make_barrier_configurations((dft_alat,
                                                                   dft_C11,
                                                                   dft_C12,
                                                                   dft_C44),
                                                                   cylinder_r=cylinder_r)

#__, __, dislo_z = disloc_ini.positions.T
#__, __, bulk_z = bulk_ini.positions.T

fix_mask = disloc_ini.constraints[0].get_indices()

x, y, z = disloc_ini.positions.T
u = disloc_ini.positions - bulk_ini.positions

disloc_ini_easy, bulk_ini_demo, u_easy = sd.make_screw_cyl(dft_alat, dft_C11, dft_C12, dft_C44,
                                                 cylinder_r=7,
                                                 cutoff=0.0,
                                                 hard_core=False)

disloc_ini_hard, bulk_ini, u_hard = sd.make_screw_cyl(dft_alat, dft_C11, dft_C12, dft_C44,
                                                 cylinder_r=7.0,
                                                 cutoff=0.0,
                                                 hard_core=True)

Wx, Wy, Wz = bulk_ini_demo.positions.T

mask = np.sqrt(Wx**2 + Wy**2) < 2.0
#print(np.count_nonzero(mask))

__, __, z = bulk_ini_demo.positions.T

#print(bulk_ini_demo)

red_mask = np.logical_or((z > 5.0 * b / 6.0 ), (z <  b / 6.0))
#print(np.count_nonzero(red_mask))
green_mask = np.logical_and((z > b / 6.0), (z < 3.0 * b / 6.0))
#print(np.count_nonzero(green_mask))
blue_mask = np.logical_and((z > 3.0*b / 6.0), (z < 5.0 * b / 6.0))
#print(np.count_nonzero(blue_mask))

Wx_bulk, Wy_bulk, Wz_bulk = bulk_ini_demo.positions.T
three_atom_mask = np.sqrt(Wx_bulk**2 + Wy_bulk**2) < 2.0

cent_x = x.min()
cent_y = y.min()

not_fixed = np.logical_not(fix_mask)
sc = ax1.scatter(x - cent_x, y - cent_y, c=u[:,2], cmap="coolwarm", label="W atoms")
cbar = plt.colorbar(sc, label="<111> displacement", pad=-0.05 )

cbar.set_ticks([u[:,2].min(), u[:,2].max()])
cbar.set_ticklabels(("-b/2", "b/2"))

core = ax1.scatter([0. - cent_x], [0. - cent_y], marker="^", c="C3", label="core position")
ax1.scatter([2.0 * alat * np.sqrt(6.)/6.0 - cent_x],
            [0. - cent_y],
            marker="^", edgecolors="C3",
            facecolor="None")

ax1.plot([-cent_x, -cent_x + 2.0 * alat * np.sqrt(6.)/6.0], [-cent_y, -cent_y], "--C3")

fixed = ax1.scatter(x[fix_mask] - cent_x,
                    y[fix_mask] - cent_y,
                    c="k", cmap="coolwarm", label="fixed atoms")

qm_radius = dft_alat*(np.sqrt(6.0)/2.0 + np.sqrt(2.0)/3.0) + 0.2
# create the circles with shapely
left = sg.Point(core_x_shift - cent_x, - cent_y).buffer(qm_radius)
right = sg.Point(- cent_x,- cent_y).buffer(qm_radius)

union = left.union(right)
union_patch = descartes.PolygonPatch(union, fc='g', ec='k',
                                     alpha=0.2, label='QM region')
patch = ax1.add_patch(union_patch)
#ax1.set_xlabel("<112> direction, $\AA$")
#ax1.set_ylabel("<110> direction, $\AA$", labelpad=1)
ax1.set_aspect("equal")

ax2_size = 100

W = ax1.scatter([], [], marker="o", s=ax2_size, edgecolor="k", facecolor="none",
                label="W atoms")

fixed = ax1.scatter([], [], marker="o", s=ax2_size, facecolor="k",
                    label="fixed atoms")

core = ax1.scatter([], [], marker="^", s=ax2_size, facecolor="C3",
                    label="core positions")

ax1.legend(loc="upper center", bbox_to_anchor=(0.5, 1.375), ncol=2,
           handles=[W, fixed, AnyObject, union_patch],
           handler_map={AnyObject: Triangles()},
           labels=["W atoms",
                   "fixed atoms",
                   "easy-core positions",
                   "QM region"])

#ax1.set_xlim((-21, 21 + core_x_shift))
#ax1.set_ylim((-21, 21))

ax1.add_artist(Circle((core_x_shift/2.0 - cent_x, 0.0 - cent_y),
                      radius=core_distance / 2.,
                      fill=False, ec="C4", ls="dashed", lw=2.0))

ax1.set_aspect("equal")
ax1.set_title("cluster configuration", loc="left")
core_x_shift = 2.0 * alat * np.sqrt(6.)/6.0
core_distance = (dislo_coord_right - dislo_coord_left)[0]

ax1.set_xlabel("<112> direction, $\mathrm{\AA}$")

ax1.xaxis.set_label_coords(0.4, 0.025)

#fig = plt.figure(figsize=(4.8, 4.0))
#ax = fig.add_subplot(111)
x, y, z = disloc_quadrupole.positions.T
#not_fixed = np.logical_not(fix_mask)

ax2 = fig.add_axes((0.06, 0.575, 0.225, 0.5))
ax2.scatter(x, y, c=final_u.T[2],
            cmap="coolwarm", label = "W atoms", vmax=1.2, vmin=-1.2)

ax2.scatter(dislo_coord_left[0],
            dislo_coord_left[1], marker="^", c="C3")

ax2.scatter(dislo_coord_left[0] + core_x_shift,
           dislo_coord_left[1],
           marker="^", edgecolor="C3", facecolor="none")

ax2.plot([dislo_coord_left[0], dislo_coord_left[0] + core_x_shift],
         [dislo_coord_left[1], dislo_coord_left[1]],
         "--C3")

ax2.scatter(dislo_coord_right[0] + core_x_shift,
           dislo_coord_right[1],
           marker="v", edgecolor="C3", facecolor="none")

ax2.plot([dislo_coord_right[0], dislo_coord_right[0] + core_x_shift],
         [dislo_coord_right[1], dislo_coord_right[1]],
         "--C3")

ax2.add_artist(Circle((dislo_coord_left[0] + core_x_shift/2.0, dislo_coord_left[1]),
                      radius=core_distance / 2.,
                      fill=False, ec="C4", ls="dashed", lw=2.0))

ax2.scatter(dislo_coord_right[0], dislo_coord_right[1], marker="v", c="C3")
#ax2.set_ylabel("<110> direction, $\AA$", labelpad=1)
ax2.set_aspect("equal")
#ax.legend(loc="upper center", bbox_to_anchor=(0.45, 1.125), ncol=2)

ax2.plot([4 * unit_cell_u[0], 5 * unit_cell_u[0] + unit_cell_v[0]],
         [0.0, 0.0], "k--")

ax2.plot([4 * unit_cell_u[0] + unit_cell_v[0],
          5 * unit_cell_u[0] + unit_cell_v[0]],
         [unit_cell_v[1], unit_cell_v[1]], "k--")

ax2.add_patch(FancyArrowPatch(posA=(5 * unit_cell_u[0] + unit_cell_v[0] * 0.8,
                                    0.0),

                              posB=(5 * unit_cell_u[0] + unit_cell_v[0] * 0.8,
                                    unit_cell_v[1]),

                              arrowstyle='<->',
                              mutation_scale=20.0, linewidth=1.5,
                              clip_on=False))


ax2.text(5 * unit_cell_u[0] + unit_cell_v[0] * 1.5,
         unit_cell_v[1] * 0.35, "%.1f $\mathrm{\AA}$" % unit_cell_v[1],
         fontsize=20)

ax2.set_xlim((-1, 48))
ax2.set_ylim((-1, 19))
ax2.set_aspect("equal")
ax2.set_title("quadrupole configuration")
#fig.savefig("quadrupole_periodic_2_NEB_circle.eps")
fig.tight_layout()

ax1.set_xlim(ax2.get_xlim())

ax1_ylim = ax1.get_ylim()

for one_ax in [ax, ax1, ax2]:

    one_ax.set_xticks([])
    one_ax.set_yticks([])


for axis in ['top','bottom','left','right']:

        ax1.spines[axis].set_color("none")
        ax2.spines[axis].set_color("none")
        ax.spines[axis].set_color("none")


ax.set_ylabel("<110> direction, $\mathrm{\AA}$")

bulk_ax = fig.add_axes((0.4, 0.65, 0.15, 0.30))

plot_bulk(bulk_ax,
          xlim=(-2.0, 4.6),
          ylim=(-1.8, 2.2))

bulk_ax.set_title("BCC bulk")

bulk_ax.set_aspect("equal")

easy_core_ax = fig.add_axes((0.4, 0.1, 0.15, 0.30))

plot_vitek(disloc_ini_easy,
           bulk_ini_demo, xyscale=2.0, plot_axes=easy_core_ax)

#easy_core_ax.set_title("easy-core")

easy_core_ax.set_xlabel("<112>")
easy_core_ax.set_ylabel("<110>")

easy_core_ax.set_aspect("equal")

easy1, = easy_core_ax.plot(0., 0., c="C3", markersize=15, marker="^", linestyle="none")
easy2, = easy_core_ax.plot([2.0 * alat * np.sqrt(6.)/6.0 ],
                           [0.],
                            marker="^", markeredgecolor="C3",
                            linestyle="none", markeredgewidth=2.0,
                            markerfacecolor="None",  markersize=15)

hard, = easy_core_ax.plot([alat * np.sqrt(6.)/6.0],
                          [alat * np.sqrt(2.)/6.0],
                           marker="*",  markeredgecolor="C3",
                           linestyle="none", markeredgewidth=2.0,
                           markerfacecolor="C3",  markersize=15, label="hard")

split, = easy_core_ax.plot([alat * np.sqrt(6.)/6.0],
                     [-alat * np.sqrt(2.)/6.0],
                     marker="x", color="C3",
                     linestyle="none",
                     markersize=15, zorder=10, markeredgewidth=4.0, label="split")

easy_core_ax.plot([ 0.0, 2.0 * alat * np.sqrt(6.)/6.0], [0.0, 0.0], "--C3")

easy_core_ax.set_xlim((-2.0, 4.6))
easy_core_ax.set_ylim((-1.8, 2.2))

proxy_hard = easy_core_ax.scatter([], [], marker="*", edgecolor="C3",
                                  facecolor="C3", s=100, lw=2)

proxy_split = easy_core_ax.scatter([], [], marker="x", color="C3",
                                   s=100, lw=3)

easy_core_ax.legend(handles=[AnyObject,
                             proxy_hard,
                             proxy_split],
                    handler_map={AnyObject: Triangles()},
                    labels=["easy-core positions",
                            "hard-core position",
                            "split-core position"],
                    loc="upper center", bbox_to_anchor=(0.5, 2.1), ncol=1)

easy_core_ax.set_yticks([])
easy_core_ax.set_xticks([])
easy_core_ax.set_title("easy-core")

for axis in ['top','bottom','left','right']:
    easy_core_ax.spines[axis].set_linewidth(02.0)


x_min, x_max = easy_core_ax.get_xlim()
width = x_max - x_min
y_min, y_max = easy_core_ax.get_ylim()
height = y_max - y_min
ax1.add_patch(mpatches.Rectangle((-cent_x + x_min, -cent_y + y_min),
                                  width, height, lw=2,
                                  linewidth=1, edgecolor='k', facecolor='none'))

ax1.plot((-cent_x + x_min + width, 73),
         (-cent_y + y_min + height, 27.25),
         linestyle="dashed", lw=2, color="k",
         clip_on=False)

ax1.plot((-cent_x + x_min + width, 73),
         (-cent_y + y_min , 6.75),
         linestyle="dashed", lw=2, color="k",
         clip_on=False)

ax1.set_xlim(ax2.get_xlim())
ax1.set_ylim(ax1_ylim)


bulk_z_ax = fig.add_axes((0.55, 0.6, 0.2, 0.40))
plot_bulk_z(bulk_z_ax)
bulk_z_ax.set_title("BCC bulk")
bulk_z_ax.yaxis.set_label_coords(-0.1, 0.625)

Wx_bulk, Wy_bulk, Wz_bulk = bulk_ini_demo.positions.T

bulk_mask = np.logical_and((Wy_bulk > -2.0), (Wy_bulk < 2.0))
total_mask = np.logical_and(red_mask, bulk_mask)

x = [-2.5, Wx_bulk[total_mask][1]]

y = [Wz_bulk[total_mask][1],
     Wz_bulk[total_mask][1]]

bulk_z_ax.plot(x, y, "k--", clip_on=False)
bulk_z_ax.plot(x, y - b, "k--", clip_on=False)

bulk_z_ax.add_patch(FancyArrowPatch(posA=(-2.25, Wz_bulk[total_mask][1] - b),
                                    posB=(-2.25, Wz_bulk[total_mask][1]),
                                    arrowstyle='<->',
                                    mutation_scale=20.0, linewidth=1.5,
                                    clip_on=False))

bulk_z_ax.text(-3.5, Wz_bulk[total_mask][1] - b/2.0 - 0.25, "b", fontsize=22)

bulk_z_with_u_ax = fig.add_axes((0.55, 0.06, 0.2, 0.40))
plot_bulk_z(bulk_z_with_u_ax)

color=["red", "lime", "blue"]

z_u = u_easy[three_atom_mask].T[2]
z_u -= z_u[0]
#zero displacement is equal to b
z_u[0] = b
Wx_bulk, Wy_bulk, Wz_bulk = bulk_ini_demo.positions.T

for b_val in [-b, 0, b]:

    for i, (x, z) in enumerate(zip(Wx_bulk[three_atom_mask],
                                   Wz_bulk[three_atom_mask])):

        bulk_z_with_u_ax.arrow(x, z + b_val, 0.0, z_u[i], width=0.15,
                               length_includes_head=True,
                               edgecolor="k", facecolor=color[i],
                               clip_on=False)
        #print(x)

bulk_z_with_u_ax.set_title("BCC bulk with u", pad=35)
bulk_z_ax.set_xlabel("")

easy_core_ax_z = fig.add_axes((0.69, 0.06, 0.2, 0.40))

hard_core_ax_z = fig.add_axes((0.8, 0.06, 0.2, 0.40))

disloc_ini_easy, bulk_ini_demo, u_easy = sd.make_screw_cyl(dft_alat,
                                                           dft_C11,
                                                           dft_C12,
                                                           dft_C44,
                                                           cylinder_r=7.0,
                                                           cutoff=0.0,
                                                           hard_core=False)


Wx_easy, Wy_easy, Wz_easy = disloc_ini_easy.positions.T
z_u = u_easy[three_atom_mask].T[2]
Wz_easy -= z_u[0]
#disloc_ini_easy.wrap()
Wx_easy, Wy_easy, Wz_easy = disloc_ini_easy.positions.T
Wx_hard, Wy_hard, Wz_hard = disloc_ini_hard.positions.T

easy_mask = np.logical_and((Wy_easy > -2), (Wy_easy < 2 ))

for b_val in [-b, 0, b]:

    total_mask = np.logical_and(red_mask, easy_mask)
    total_mask = np.logical_and(total_mask, three_atom_mask)

    easy_core_ax_z.scatter(Wx_easy[total_mask],
                           Wz_easy[total_mask] + b_val + b,
                           color="red",
                           edgecolor="k", s=size, lw=1, clip_on=False)

    x = [-8.8, Wx_easy[total_mask]]
    y = [Wz_easy[total_mask] + b_val + b,
         Wz_easy[total_mask] + b_val + b]

    easy_core_ax_z.plot(x, y, "r--", clip_on=False, lw=1)

    delta = Wz_bulk[total_mask] -  Wz_hard[total_mask]

    hard_core_ax_z.scatter(Wx_hard[total_mask],
                           Wz_hard[total_mask] + b_val + delta,
                           color="red",
                           edgecolor="k", s=size, lw=1)


    total_mask = np.logical_and(green_mask, easy_mask)
    total_mask = np.logical_and(total_mask, three_atom_mask)
    easy_core_ax_z.scatter(Wx_easy[total_mask],
                           Wz_easy[total_mask] + b_val,
                           color="lime",
                           edgecolor="k", s=size, lw=1, clip_on=False)

    x = [-6.3, Wx_easy[total_mask]]
    y = [Wz_easy[total_mask] + b_val,
         Wz_easy[total_mask] + b_val]

    easy_core_ax_z.plot(x, y, "--", color="lime", clip_on=False, lw=1)

    hard_core_ax_z.scatter(Wx_hard[total_mask],
                           Wz_hard[total_mask] + b_val + delta,
                           color="lime",
                           edgecolor="k", s=size, lw=1)

    total_mask = np.logical_and(blue_mask, easy_mask)
    total_mask = np.logical_and(total_mask, three_atom_mask)

    easy_core_ax_z.scatter(Wx_easy[total_mask],
                           Wz_easy[total_mask] + b_val,
                           color="blue",
                           edgecolor="k", s=size, lw=1,
                           clip_on=False)

    x = [-7.5, Wx_easy[total_mask]]
    y = [Wz_easy[total_mask] + b_val,
         Wz_easy[total_mask] + b_val]

    easy_core_ax_z.plot(x, y, "--", color="blue", lw=1,
                        clip_on=False, zorder=-1,)

    hard_core_ax_z.scatter(Wx_hard[total_mask],
                           Wz_hard[total_mask] + b_val + delta,
                           color="blue",
                           edgecolor="k", s=size, lw=1)


easy_core_ax_z.set_xlim(bulk_z_with_u_ax.get_xlim())
easy_core_ax_z.set_ylim(bulk_z_with_u_ax.get_ylim())

easy_core_ax_z.set_aspect("equal")
easy_core_ax_z.set_xlabel("<112>")
easy_core_ax_z.set_ylabel("<111>", labelpad=-100)


easy_core_ax_z.set_title("easy-core", pad=45)

hard_core_ax_z.set_xlim(bulk_z_with_u_ax.get_xlim())
hard_core_ax_z.set_ylim(bulk_z_with_u_ax.get_ylim())

hard_core_ax_z.set_aspect("equal")
hard_core_ax_z.set_xlabel("<112>")
#hard_core_ax_z.set_ylabel("<111>", labelpad=-125)


hard_core_ax_z.set_title("hard-core", pad=45)



for one_ax in [bulk_ax, bulk_z_ax, bulk_z_with_u_ax, easy_core_ax_z, hard_core_ax_z]:

    one_ax.set_xticks([])
    one_ax.set_yticks([])

    for axis in ['top','bottom','left','right']:
        one_ax.spines[axis].set_color("none")

legend_ax = fig.add_axes((0.75, 0.625, 0.2, 0.40))

W = legend_ax.scatter([], [], marker="o", c="white", s=150,
                      edgecolors="k", label="W atoms")
#z_border = legend_ax.axhline(100.0, linestyle="dashed", c="c",
#                             label="PBC in <111> direction")
handles = [W]
tmp = legend_ax.scatter(0.0, 100.0, #linestyle="solid",
                        c="k", marker=r'$\rightarrow$',
                        s=850.0,
                        label="Differential <111> dislacement")
handles.append(tmp)

for color in ["red", "lime", "blue"]:

    tmp = legend_ax.scatter(0.0, 100.0, #linestyle="solid",
                            c=color, marker=r'$\rightarrow$',
                            edgecolor="k", s=850.0,
                            label="<111> displacement")

    handles.append(tmp)

legend_ax.set_ylim((0.0, 1.0))

legend_ax.legend(handles=handles, loc="center", fontsize=14)

legend_ax.set_xticks([])
legend_ax.set_yticks([])

for axis in ['top','bottom','left','right']:
    legend_ax.spines[axis].set_linewidth(0.0)
    legend_ax.spines[axis].set_color("white")

ax1.text(-3, 35, "(d)", fontsize=22)
ax2.text(-3, 12.5, "(a)", fontsize=22)
bulk_ax.text(-3.0, 2.5, "(b)", fontsize=22)
bulk_z_ax.text(-3.5, 4.0, "(c)", fontsize=22)
bulk_z_with_u_ax.text(-4.0, 4.0, "(f)", fontsize=22)
easy_core_ax.text(-3.25, 2.75, "(e)", fontsize=22)
#easy_core_ax_z.text(2.5, 4.5, "(g)", fontsize=22)

#plt.show()
fig.savefig("cluster_quad_core_structure.pdf")
