import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from plot_tools import *
from scipy.optimize import curve_fit
import GraphSettings

GraphSettings.Apply_thesis_settings()

#quad_du_df = pd.read_csv("quad_du.csv")
#cyl_du_df = pd.read_csv("cyl_du_comparison.csv")
#quad_du_df = quad_du_df.drop(6)
#cyl_du_df = cyl_du_df.drop(6)

df = pd.read_csv("size_comparison_new.csv")
# drop the last configuration
df = df.iloc[:-1]

fig, ax = plt.subplots(figsize=(7,6))


#ax.loglog(df.core_distance, df.rel_delta_ref_quad_single_barrier,
#        "s-", label="quadrupole single")

markersize = 10

cyl_energy, = ax.loglog(df.core_distance, df.rel_delta_ref_cyl_barrier,
                       "o-", label="cluster", markersize=markersize)

quad_energy, = ax.loglog(df.core_distance, df.rel_delta_ref_quad_double_barrier,
                "^-", label="quadrupole", markersize=markersize)


"""
quad_du, = ax.loglog(df.core_distance, quad_du_df.Du_local[:-1],
                    "s-", label="quadrupole", markersize=markersize,fillstyle="none")

cyl_du, = ax.loglog(df.core_distance, cyl_du_df.Du_local[:-1],
                   "s-", label="cylinder", markersize=markersize, fillstyle="none")
"""
ax.set_xlabel("Core distance/Cluster diameter ($\mathrm{\AA}$)")

energy_title = r"$|E_\mathrm{glide} - E^\mathrm{ref}_\mathrm{glide}|/E^\mathrm{ref}_\mathrm{glide}$"

ax.set_ylabel(energy_title)

first_legend = ax.legend(loc="lower left",
                         handles=[cyl_energy, quad_energy])#,
                         #title=energy_title)

ax.add_artist(first_legend)
"""
ax.legend(loc="upper right", handles=[quad_du, cyl_du]),
          title=r"$|\nabla u_R - \nabla u_{ref}|_{local}$")

"""
ax.grid(True, ls="dashed")

ax.get_xaxis().set_tick_params(which='minor', size=0)
ax.get_xaxis().set_tick_params(which='minor', width=0)

ax.set_xticks(df.core_distance)
ax.set_xticklabels(["%i"%int(D) for D in df.core_distance.round()])
#ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())

top_ax = ax.twiny()
top_ax.set_xscale("log")
top_ax.set_xlim(ax.get_xlim())
top_ax.set_xticks(ax.get_xticks())
top_ax.set_xticklabels(df.cyl_Nat)
top_ax.set_xlabel("Number of atoms in cluster cell")
top_ax.get_xaxis().set_tick_params(which='minor', size=0)
top_ax.get_xaxis().set_tick_params(which='minor', width=0)

for tick in top_ax.get_xticklabels()[:-2]:
    tick.set_rotation(60)

x = df.core_distance[2:].values
y = df.rel_delta_ref_cyl_barrier[2:].values
p = -2.0
offset = 1.7

func = lambda X, a, p: a*X**p
popt, pcov = curve_fit(func, x, y)

a, p = popt

ax.plot(x, offset * func(x, *popt), 'k--')

xmiddle = np.exp((np.log(x[0]) + np.log(x[-1]))/2)
ax.text(xmiddle, offset*func(xmiddle, *popt), r'$R^{%.1f}$' % p, va='bottom')

x = df.core_distance[::2].values
y = df.rel_delta_ref_quad_double_barrier[::2].values
p = -2.0


func = lambda X, a, p: a*X**p
popt, pcov = curve_fit(func, x, y)

a, p = popt

offset = 1.4

ax.plot(x[1:], offset * func(x[1:], *popt), 'k--')

xmiddle = np.exp((np.log(x[0]) + np.log(x[-1]))/2)
ax.text(xmiddle, offset*func(xmiddle, *popt), r'$R^{%.1f}$' % p, va='bottom')

fig.tight_layout()

'''
df = pd.read_csv("size_comparison_old.csv")

cyl_energy, = ax.loglog(df.core_distance, df.rel_delta_ref_cyl_barrier,
                       "o-", label="cluster", markersize=markersize)

quad_energy, = ax.loglog(df.core_distance, df.rel_delta_ref_quad_double_barrier,
        "o-", label="quadrupole", markersize=markersize)
'''
fig.savefig("quad_vs_cyl_eam3.pdf")
